/*******************************************************************************
 * PVGA1A framebuffer support
 *******************************************************************************
 *
   Copyright (C) 1998, Brian S. Julin bri@calyx.com
   Copyright (C) 1997, The Linux-GGI Project
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *
 *******************************************************************************
 */

#define VGA_WINDOW_SIZE (64 * 1024)

#include <kgi/maintainers.h>
#define	MAINTAINER	Brian_S_Julin
#define DRIVER_NAME	"PVGA1A graphics driver"
#define	DRIVER_REV	"$Revision: 1.7 $"

#define DEBUG_LEVEL	0

/* KGI includes */
#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#include "pvga1a_suggest.c"
#include "pvga1a_mmio.c"

#include "WD/pvga1a_io.h"

/* KGI module interface */

int kgim_accel_init(struct kgi_card *card, struct kgi_display *dpy)
{
	/* FIXME: We should really do something here! */
	NOTICE1("PVGA1A Graphics Driver loaded");
	return EOK;
}

void kgim_accel_done(struct kgi_display *dpy)
{
	NOTICE1("PVGA1A Graphics Driver removed");
}

int kgim_accel_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
			      enum kgi_tm_cmd *cmd)
{
	/* !!! Should we check for accel-friendly dimensions here ??? */
	switch (*cmd) {
		case CMD_PROPOSE:
		NOTICE1("Setting up mmio");
		return graphics_mmio_setup(dpy, mode);
		default:
		break;
	}
	
	return EOK;
}

void kgim_accel_set_mode (struct kgi_display *dpy, struct kgi_mode *mode)
{
  int lockval;
  /* Reset all bank offsets to 0 */
  outPR0A(0);
  outPR0B(0);
  /* Enable split bank */
  lockval = rwunlockGRC();
  switch(mode->request.graphtype) {
  case KGIGT_8BIT:
    /*    if ((mode->request.virt.x * mode->request.virt.y) > VGA_WINDOW_SIZE) */
    outbitsGRC(GR0B_PR0B_ENAB, GR0B_PR0B_ENAB, GRC_MEMORY); 
      /*    else
	    outbitsGRC(0, GR0B_PR0B_ENAB, GRC_MEMORY); */
    break;
  case KGIGT_TEXT16:
  case KGIGT_TEXT32:
  case KGIGT_1BIT:
  case KGIGT_4BIT:
  default:
    outbitsGRC(0, GR0B_PR0B_ENAB, GRC_MEMORY); 
    break;
  }
  outbGRC(lockval, GRC_UPGRC_LOCK);
}

int kgim_accel_command (struct kgi_graphic *gr, kgi_uint cmd, kgi_uint arg)
{ 
	kgi_suggest *sugg;
	const char *name;

	switch (cmd) {
		case ACCEL_GETSUGGEST:
		sugg = (kgi_suggest *)(gr->io_buf);
		gr->io_res = gr->io_buf;
		
		if (sugg->id < 0 || sugg->id > 1)
			return -E(ACCEL, BOGUS);
		if (sugg->id == 1) { /* Generic RAMDAC support */
			strcpy(sugg->name, "generic-ramdac");
			sugg->args[0] = 0;
			sugg->id = 0;
			return 0;
		}
		switch (gr->dev.mode.graphtype) {
			case KGIGT_TEXT16:
			name = sugg_default[SG_TEXT16];
			break;
			case KGIGT_1BIT:
			name = sugg_default[SG_1BIT];
			break;
			case KGIGT_4BIT:
			name = sugg_default[SG_4BIT];
			break;
			case KGIGT_8BIT:
			  if ((gr->dev.mode.virt.x * 
			       gr->dev.mode.virt.y) <= VGA_WINDOW_SIZE)
			    /*	if (gr->dev.mode.visible.x <= 320 &&
			    gr->dev.mode.visible.y <= 200) */
			    name = sugg_default[SG_8BIT];
			  else
			    name = sugg_default[SG_8BIT_MMAP];
			break;
			default:
			return -E(ACCEL, BOGUS);
		}
		strcpy(sugg->name, name);
		sugg->args[0] = 0;
		sugg->id = 1;	/* Next is RAMDAC */
		return 0;
		default:
		break;
	}
			
	return -E(ACCEL, NOSUP_ALWAYS_MMAP);
}

